function costs = preselectionfeaturesfromvoting(left, right, row, dispsofpreviousrow, params)
%display('Running preselectionfeaturesfromvoting');
%Use a houghline voting scheme for disparity calcalation of an entire image
%using a single line from newpixelfeaturesentireimage. 
%Accumulate pixels of the the images according to windowlength
%Select one pixel candidate at w from left image
%Select the right image cut out from w-D to w
%At each w, fill an two column R-Table with the values of right image
%cutout and the corresponding indices
%Build a AD-vector abs(leftpixel - right image cut out)
%Find minimum in AD-Vector
%Vote in Houghline at minimum index (currentcol - w) + index of Rtable;
%Three approaches:
%   Use one single minimum
%   Use all minima below a certain threshold for voting
%   Use a hard count of minima
%Repeat for according blockheight of lines and sum up the houghline
%Find maximum in houghline for each block and use it as (dominating) disparity of that
%block
%Try to refine disparities: Analyse houghline for a number of maxima
%In function disparityplausibility: Check each pixel of sum images for best
%fit of each disparity of found maxima. 

%RETURNS a single line of costs!

%For debugging:
if nargin == 0
    clear VARIABLES;
    %Kitti
    readpath = 'pathtoimages';
    dataset = '000026';
    leftimagepath = [readpath,dataset,'\left.png'];  
    rightimagepath = [readpath,dataset,'\right.png'];
    left = imread(leftimagepath);
    right = imread(rightimagepath);
    starty = 200; startx = 500;
    left = left(starty : starty + 31, startx : startx + 31);
    right = right(starty : starty + 31, startx : startx + 31);
    row = 8;
    params.numberOfDisparities = 16;
    params.CTlength = 3;
    params.CTheight = 3;
    params.P2 = 144;
    dispsofpreviousrow = zeros(32,1);
    display(['Calculating disparities for dataset ', dataset])
    % scalefactor = 256;    %Kitti: 256
    % thres = 3; 
    %D = 160; %Kitti: 110
    %tolerance = 3;
end

D = params.numberOfDisparities;
windowlength = 1;   %For the AD-Images
windowheight = 1;   %For the AD-Images
blockheight = 1;
blockwidth = D; 

numberofmaxima = 3; % Number of maxima in houghlines, i.e. the number of most voted disparities 

leftimage = double(left);
rightimage = double(right);

[hil, wil] = size(leftimage);

if (mod(windowlength,2) == 0) || (mod(windowheight,2) == 0)
    error('Window sizes must both be odd!');
end

numberofblocks = ceil(wil / D);
numberofminima = 3;     %Used for minima hard count 


endline = row + blockheight - 1;

if( (endline + floor(windowheight/2) > hil) || (row - floor(windowheight/2) <= 0) )
	error('Window too high or too low for this specific line...');
else
	linerange = row : endline;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Get cut-out of  images
leftstripe = leftimage(linerange,:);
rightstripe = rightimage(linerange,:);
houghlinesingle = zeros(wil,1);
houghlinehardcount = zeros(wil,1);

for h = 1 : blockheight
	currentcol = D;
	for w = 1 : wil
		if (w > currentcol)
			currentcol = currentcol + D;
		end
		if (currentcol > wil)
			currentcol = wil;
		end
		
		wend = w;
		wstart = max(w - D + 1, 1);
		
		values = rightstripe(h, wstart : wend);   %Values to match (left search range)
		%indices = wstart : wend;     %Indices /wo offset
	
		currentpixel = leftstripe(h,w);
		currentsadvector = abs(values - currentpixel);     %SAD between pixel and search range

		[val,ind] = min(currentsadvector);
		
		%votecand_old = (currentcol - w) + rtable(ind,2);
		%display(currentcol);
		votecand = (currentcol - w) + wstart + ind - 1;
		
		%Use single minimum
		if votecand >= 1 && votecand <= wil
			houghlinesingle(votecand) = houghlinesingle(votecand) + 1;
		end
            
		%Use multiple minima defined by a threshold:
%             th = val*1.1;     %Threshold
%             for i = 1 : length(currentsadvector)
%                 if(currentsadvector(i) < th)
%                     votecand = currentcol - w + rtable(i,2);
%                     if votecand >= 1 && votecand <= wil
%                         houghlinehardthres(votecand) = houghlinehardthres(votecand) + 1;
%                     end
%                 end
%             end
		
		%Use multiple minima defined by a hard number of minima:
		for i = 1 : numberofminima
			[~,ind] = min(currentsadvector);
			%votecand = (currentcol - w) + rtable(ind,2);
			votecand = (currentcol - w) + wstart + ind - 1;
			if votecand >= 1 && votecand <= wil
				houghlinehardcount(votecand) = houghlinehardcount(votecand) + 1;

			end
			currentsadvector(ind) = Inf;    %Eliminate processed minimum
		end
	   
	end
end
    
    
%Get disparities of each block except last block. Here, ind must be subtracted from wil
%     disphardcount = zeros(numberofblocks,2);
%     for d = 1 : numberofblocks - 1
%        [val,ind] = max(houghlinehardcount( (d-1)*D + 1 : (d-1)*D + D));
%        disphardcount(d,1) = D - ind;
%        disphardcount(d,2) = val;
%     end
%     %Set disp of last block:
%     [val,ind] = max(houghlinehardcount( d*D + 1 : wil));
%     disphardcount(d+1,1) = wil - (d*D+1) - ind + 1;
%     disphardcount(d+1,2) = val;

    %Generate a disparitiy line from the disparities of each block:
    %displinehardcount = zeros(wil,1);
    %for i = 0 : numberofblocks - 2
    %    displinehardcount(i*D+1 : (i+1)*D) = disphardcount(i+1,1);
    %end
    %displinehardcount((numberofblocks-1)*D+1 : wil) = disphardcount(numberofblocks,1);
    %dispmapHardcountAsis(linecounter,:) = displinehardcount;

bestindicessingle = zeros(numberofblocks,numberofmaxima);
bestvaluessingle = zeros(numberofblocks,numberofmaxima);
for d = 1 : numberofblocks
   [val,ind] = sort(houghlinesingle( (d-1)*D + 1 : min( ((d-1)*D + D),wil) ));
   bestindicessingle (d,:) = ind(end-numberofmaxima+1:end);     %Select the best numberofmaxima values
   bestvaluessingle (d,:) = val(end-numberofmaxima+1:end);
end
[~,costs] = disparityplausibilitynew2PROD_SGMcosts(leftimage,rightimage,row,bestindicessingle,dispsofpreviousrow, params);

%Uncomment for hough line hard count:
%     bestindiceshardcount = zeros(numberofblocks,numberofmaxima);
%     bestvalueshardcount = zeros(numberofblocks,numberofmaxima);
%     for d = 1 : numberofblocks
%        [val,ind] = sort(houghlinehardcount( (d-1)*D + 1 : min( ((d-1)*D + D),wil) ));
%        bestindiceshardcount(d,:) = ind(end-numberofmaxima+1:end);
%        bestvalueshardcount (d,:) = val(end-numberofmaxima+1:end);
%     end
% 
%     [~,costs] = disparityplausibilitynew2PROD_SGMcosts(leftimage,rightimage,row,bestindiceshardcount,dispsofpreviousrow, params);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Cost calulation done!

%Some post proc, i.e. Median Filtering:
%%
% ownmap = zeros(hil,wil,6);
% ownmap(:,:,1) = dispmapSingleAsis;
% ownmap(:,:,2) = dispmapSingleRefined;
% ownmap(:,:,3) = dispmapHardthresAsis;
% ownmap(:,:,4) = dispmapHardthresRefined;
% ownmap(:,:,5) = dispmapHardcountAsis;
% ownmap(:,:,6) = dispmapHardcountRefined;
% 
% filtmap = zeros(hil, wil, 6);
% for i = 1 : 6
%     filtmap(:,:,i) = medfilt2(ownmap(:,:,i),[filterheight filterlen],'symmetric');
% end

% gtAll = imread([readpath,dataset,'\eval\gtAll.png']);    
% gtNoc = imread([readpath,dataset,'\eval\gtNoc.png']);
% 
% percentageAll = zeros(1,6); percentageNoc = zeros(1,6);
% 
% for i = 1 : 6
%     percentageAll(i) = disp_error(double(gtAll)/scalefactor, ownmap(:,:,i),tolerance) * 100;   %disp_error is from KITTI devkit
%     percentageNoc(i) = disp_error(double(gtNoc)/scalefactor, ownmap(:,:,i),tolerance) * 100;
% end

% display([char(10),'Error percents of the approaches: All, Noc (NOT FILTERED)']); 
% for i = 1 : 6
%     result = [num2str(percentageAll(i)), char(9), num2str(percentageNoc(i))];
%     display(['Approach ', num2str(i), ': ', result]);
% end
% 
% percentageAll = zeros(1,6); percentageNoc = zeros(1,6);
% for i = 1 : 6
%     percentageAll(i) = disp_error(double(gtAll)/scalefactor, filtmap(:,:,i),tolerance) * 100;   %disp_error is from KITTI devkit
%     percentageNoc(i) = disp_error(double(gtNoc)/scalefactor, filtmap(:,:,i),tolerance) * 100;
% end


% display([char(10),'Error percents of the approaches: All, Noc (FILTERED)']); 
% for i = 1 : 6
%     result = [num2str(percentageAll(i)), char(9), num2str(percentageNoc(i))];
%     display(['Approach ', num2str(i), ': ', result]);
% end


%%
%SAve the disparity maps separately for further postprocessing
%save([writepath,'/filterdisps.mat'],'dispmapSingleAsis', 'dispmapSingleRefined', 'dispmapHardthresAsis',...
%    'dispmapHardthresRefined','dispmapHardcountAsis','dispmapHardcountRefined');

% run middleburyevalfilter.m;
%run kittievalfilter.m;



%display('DONE');

end %of function

