----------------------------------------------------------------------------------
-- Company: 
-- Engineer: schfra
-- 
-- Create Date:    11:39:20 03/28/2014 
-- Design Name: 
-- Module Name:    ad - Behavioral 
-- Project Name: absolute_differences 
-- Target Devices: Virtex 5 FX 130T
-- Tool versions: ISE 12.1
-- Description: On each clock cycle, receives a left and right pixel. Clocks the right 
--				pixels into an input pixel pipeline with length D. When the pipeline is full, use signed math to 
--				build the difference between current left pixel and all D right pixels in the pipeline in parallel
--				Writes the D results as unsigned values into advector output signal which is assigned to output advector. 
--				at time x, leftin - right(x-D ... x - 1) is calulated, i.e. leftin(17) - rightpipe(1 .. 16)				
--
-- Dependencies: Package constantsandtypesPack in file constantsandtypes.vhd
-- Synthesis: D=160, [hi,wi]=[375,1242], 368 MHz, 2571 Reg, 2898 LUTs
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;

LIBRARY work;
USE work.allconstantsPack.ALL;


ENTITY ad_superpipe IS
PORT(
	clk : in std_logic;
	reset : in std_logic;
	output_valid : out std_logic;
	leftin : in unsigned(PixelBitWidth - 1 downto 0);
	rightin : in unsigned(PixelBitWidth - 1 downto 0);
	colc : in unsigned(ImageWidthInBits - 1 downto 0);
	advector : out pixelBitWidthpipeLenDType
);
END ad_superpipe;

ARCHITECTURE Behavioral OF ad_superpipe IS

--signal colc, colc_next : unsigned(ImageWidthInBits - 1 downto 0);

TYPE superpipeType IS ARRAY(NATURAL RANGE 1 TO D+1) of signed(PixelBitWidth downto 0);	--9 Bit
signal superpipe, superpipe_next : superpipeType;

signal leftinDelay, leftinDelay_next : unsigned(PixelBitWidth - 1 downto 0);

signal oe, oe_next : std_logic;

BEGIN

clkProc : PROCESS(reset, clk)
BEGIN
	IF reset = '1' THEN
		superpipe <= (OTHERS => (OTHERS => '0'));
		--colc <= to_unsigned(1,ImageWidthInBits);
		leftinDelay <= (OTHERS => '0');
		oe <= '0';
		
	ELSIF clk'event AND clk = '1' THEN
		superpipe <= superpipe_next;
		--colc <= colc_next;
		leftinDelay <= leftinDelay_next;
		oe <= oe_next;
		
	END IF;
END PROCESS;

combProc : PROCESS(leftin, leftinDelay, rightin, colc, superpipe, oe)
variable leftdiff : signed(PixelBitWidth downto 0) := (OTHERS => '0');	--Difference of left pixel from diff = l(n) - l(n-1)
--9 Bit
BEGIN

	oe_next <= oe;
	
	IF colc > D THEN
		leftinDelay_next <= leftin;
		oe_next <= '1';	--enable output 
	ELSE
		leftinDelay_next <= (OTHERS => '0');
		oe_next <= '0';
	END IF;
	
	superpipe_next <= superpipe;
	
	IF colc >= D + 1 THEN
		leftdiff := to_signed(to_integer(leftin) - to_integer(leftinDelay), PixelBitWidth + 1);	-- -255	... +255, Conversion to 9 Bits
		FOR i IN 1 TO D LOOP
			superpipe_next(i) <= leftdiff + superpipe(i+1);	-- -255 ... +255 + -255 ... 255 = -510 ... 510 = 10 Bit signed
		END LOOP;
		
		superpipe_next(D+1) <= ('0' & signed(leftin)) - ('0' & signed(rightin));
		--superpipe_next(D+1) <= ('0' & '0' & signed(leftin)) - ('0' & '0' & signed(rightin));
		
	ELSE
		leftdiff := '0' & signed(leftin);	-- leftdiff: 9 Bits, leftin 8 Bits. Result: 0 ... +255
		--superpipe_next(D+1) <= to_signed(0 - to_integer(rightin), PixelBitWidth + 2);	-- -255 ... 0
		superpipe_next(D+1) <= to_signed(0 - to_integer(rightin), PixelBitWidth + 1);	-- -255 ... 0
		superpipe_next(1 TO D) <= superpipe(2 TO D+1);
	END IF;
	
	--IF colc < IMAGEWIDTH THEN
	--	colc_next <= colc + 1;
	--ELSE
	--	colc_next <= to_unsigned(1,ImageWidthInBits);
	--END IF;
	
END PROCESS;

--Assign signed pipe values to unsigned output vector:
--Problem: Because of one cycle delay, the last result of a line is not displayed, when using colc check!
output : PROCESS(superpipe, oe)
variable tmp : signed(PixelBitWidth downto 0) := (OTHERS => '0');	--10 Bit
variable tmp2 : unsigned(PixelBitWidth - 1 downto 0) := (OTHERS => '0');
BEGIN
	
--	IF oe = '1' THEN
--		FOR i IN 1 TO D LOOP
--			tmp := superpipe(i);
--			IF tmp(PixelBitWidth) = '1' THEN
--			--Check signed-Bit (First bit in signed-vector)
--				advector(i) <= unsigned(NOT(tmp(PixelBitWidth - 1 downto 0))) + 1;
--			ELSE
--				advector(i) <= unsigned(tmp(PixelBitWidth - 1 downto 0));
--			END IF;
--		
--		END LOOP;
--	ELSE
--		advector <= (OTHERS => (OTHERS => '1'));
--	END IF;
	
	--ALternativ statt eines 9 Bit Addierers kann ein 9 Bit Subtrahierer genutzt werden. Knnte effizienter sein...:
	IF oe = '1' THEN
		FOR i IN 1 TO D LOOP
			tmp := superpipe(i);
			IF tmp(PixelBitWidth) = '1' THEN
				tmp2 := unsigned(tmp(PixelbitWidth - 1 downto 0)) - 1;
				advector(i) <= NOT(tmp2);
			ELSE
				advector(i) <= unsigned(tmp(PixelBitWidth - 1 downto 0));
			END IF;
		END LOOP;
	ELSE
		advector <= (OTHERS => (OTHERS => '1'));
	END IF;
		

END PROCESS;

output_valid <= oe;

END Behavioral;


















