----------------------------------------------------------------------------------
-- Company: 
-- Engineer: schfra
-- 
-- Create Date:    11:39:20 03/28/2014 
-- Design Name: 
-- Module Name:    ad - Behavioral 
-- Project Name: absolute_differences 
-- Target Devices: Virtex 5 FX 130T
-- Tool versions: ISE 12.1
-- Description: On each clock cycle, receives a left and right pixel. Clocks the right 
--				pixels into an input pixel pipeline with length D. When the pipeline is full, use signed math to 
--				build the difference between current left pixel and all D right pixels in the pipeline in parallel
--				Writes the D results as unsigned values into advector output signal which is assigned to output advector. 
--				at time x, leftin - right(x-D ... x - 1) is calulated, i.e. leftin(17) - rightpipe(1 .. 16)				
--
-- Dependencies: Package constantsandtypesPack in file constantsandtypes.vhd
-- Synthesis: D=160, [hi,wi]=[375,1242], 368 MHz, 2571 Reg, 2898 LUTs
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;

LIBRARY work;
USE work.allconstantsPack.ALL;

ENTITY ad_twopipes IS
PORT(
	clk : in std_logic;
	reset : in std_logic;
	output_valid : OUT std_logic;
	leftin : in unsigned(PixelBitWidth - 1 downto 0);
	rightin : in unsigned(PixelBitWidth - 1 downto 0);
	colc : in unsigned(ImageWidthInBits - 1 downto 0);
	advector : out pixelBitWidthpipeLenDType
);
END ad_twopipes;

ARCHITECTURE Behavioral OF ad_twopipes IS

--signal colc, colc_next : unsigned(ImageWidthInBits - 1 downto 0);
signal advectorSig, advectorSig_next : pixelBitWidthpipeLenDType;
signal rightpipe, rightpipe_next : pixelBitWidthpipeLenDType;

BEGIN

clkProc : PROCESS(reset, clk)
BEGIN
	IF reset = '1' THEN
		advectorSig <= (OTHERS => (OTHERS => '0'));
		rightpipe <= (OTHERS => (OTHERS => '0'));
		output_valid <= '0';
		--colc <= to_unsigned(1,ImageWidthInBits);
	ELSIF clk'event AND clk = '1' THEN
		advectorSig <= advectorSig_next;
		rightpipe <= rightpipe_next;
		output_valid <= '0';
		--colc <= colc_next;
	END IF;
END PROCESS;

combProc : PROCESS(leftin, rightin, advectorSig, rightpipe, colc)
variable temp : signed(PixelBitWidth downto 0) := (OTHERS => '0');
--variable smaller, bigger : unsigned(PixelBitWidth - 1 downto 0) := (OTHERS => '0');
BEGIN
	
	rightpipe_next(D) <= rightin;
	rightpipe_next (1 TO D - 1) <= rightpipe(2 TO D);
	advectorSig_next <= advectorSig;
	
	
	--IF colc < IMAGEWIDTH THEN
	--	colc_next <= colc + 1;
	--ELSE
	--	colc_next <= to_unsigned(1,ImageWidthInBits);
	--END IF;
	
	IF colc >= D + 1 THEN
	
		--Calc absolute differences between left and right pipe to fill sad-vector from 2...D
		--use signed math to save the big comparations:
		--Build signed difference. Then, check sign bit (temp(8)). If result is negative, convert it to unsigned
		FOR i IN 1 TO D LOOP
			temp := to_signed(to_integer(leftin) - to_integer(rightpipe(i)), PixelBitWidth + 1);
			
			IF temp(PixelBitWidth) = '1' THEN
				advectorSig_next(i) <= (NOT(unsigned(temp(PixelBitWidth - 1 downto 0)))) + 1;		--Result is negative
			ELSE
				advectorSig_next(i) <= unsigned(temp(PixelBitWidth - 1 downto 0));
			END IF;
		END LOOP;
	
		--Alternativ (Bentigt allerdings mehr LUTs bei gleicher Anzahl Reg und ist langsamer... Mp):
--		FOR i IN 1 TO D LOOP
--			IF leftin > rightpipe(i) THEN
--				bigger := leftin;
--				smaller := rightpipe(i);
--			ELSE
--				bigger := rightpipe(i);
--				smaller := leftin;
--			END IF;
--			advectorSig_next(i) <= bigger - smaller;
--		END LOOP;
		
		--Calc absolute differences between left and right for sad-vector(1)
--		temp := to_signed(to_integer(leftin) - to_integer(rightin), PixelBitWidth + 1);
--		IF temp(PixelBitWidth) = '1' THEN
--			advectorSig_next(1) <= (NOT(unsigned(temp(PixelBitWidth - 1 downto 0)))) + 1;	--result is negative, i.e. twos complement. 
--		ELSE
--			advectorSig_next(1) <= unsigned(temp(PixelBitWidth - 1 downto 0));
--		END IF;
	END IF;

END PROCESS;

advector <= advectorSig;

END Behavioral;


















