--instantiates census modules and clocks census values of right pixel stream in a pipeline

library ieee;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library work;
use work.allconstantsPack.ALL;


entity censusLR is
	GENERIC(
		IMAGEWIDTH : integer := 1024;
		DATAWIDTH : integer := 8
	);
	port(
		clk : IN std_logic;
		reset : IN std_logic;
		enable : IN std_logic;
		ready : out std_logic;
		leftin : IN unsigned(7 downto 0);
		rightin : IN unsigned(7 downto 0);
		leftout : OUT censusVectorType;
		rightout : OUT censusVectorType  
	);
end censusLR;

architecture Behavioral of censusLR is

	COMPONENT census_MxN
		generic(
			IMAGEWIDTH : integer := IMAGEWIDTH;	--The census transform clockes in 2xIMAGEWIDTH + WINDOWLENGTH pixels before trasnform
			DATAWIDTH : integer := DATAWIDTH
		);
		PORT(
			clk : in std_logic;
			reset : in std_logic;
			datain : in unsigned(7 downto 0);
			valid : out std_logic;
			targetrow : out unsigned(ImageHeightInBits - 1 downto 0);
			targetcol : out unsigned(ImageWidthInBits - 1 downto 0);
			censusout : out unsigned(NumberOfCensusValues - 1 downto 0)
		);
	END COMPONENT;

	signal targetrowL, targetrowR : unsigned(ImageHeightInBits - 1 downto 0);
	signal targetcolL, targetcolR : unsigned(ImageWidthInBits - 1 downto 0);
	
	signal censusRpipe, censusRpipe_next : censusVectorType;
	signal censusLpipe, censusLpipe_next : censusVectorType;
	
	signal censusLsig, censusRsig : unsigned(NumberOfCensusValues - 1 downto 0);
	
	signal validLsig, validRsig : std_logic;

begin

	censusL : census_MxN 
	PORT MAP(
		clk => clk,
		reset => reset,
		datain => leftin,
		valid => validLsig,
		targetrow => targetrowL,
		targetcol => targetcolL,
		censusout => censusLsig
	);
	
	censusR : census_MxN
	PORT MAP(
		clk => clk,
		reset => reset,
		datain => rightin,
		valid => validRsig,
		targetrow => targetrowR,
		targetcol => targetcolR,
		censusout => censusRsig
	);
	
	--Clocked process
	clkProc : PROCESS(clk, reset)
	begin
		IF reset = '1' then
			ready <= '0';
			censusRpipe <= (OTHERS => (OTHERS => '0'));
			censusLpipe <= (OTHERS => (OTHERS => '0'));
			--colc <= to_unsigned(1,ImageWidthInBits);
			--rowc <= to_unsigned(1,ImageHeightInBits);	
			
			
		ELSIF (clk'event AND clk = '1') THEN
			IF enable = '1' THEN
				ready <= '1';
				censusRpipe <= censusRpipe_next;
				censusLpipe <= censusLpipe_next;
				--colc <= colc_next;
				--rowc <= rowc_next;
			END IF;
		END IF;
	END PROCESS clkProc;
	
	combProc : PROCESS(censusRsig, censusLsig, censusRpipe, censusLpipe)
	BEGIN
		censusRpipe_next(D) <= censusRsig;
		censusRpipe_next(1 TO D-1) <= censusRpipe(2 TO D);
		
		censusLpipe_next(D) <= censusLsig;
		censusLpipe_next(1 TO D-1) <= censusLpipe(2 TO D);
	
	END PROCESS;
	
	leftout <= censusLpipe;
	rightout <= censusRpipe;


end Behavioral;



















