--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.ALL;
USE IEEE.math_real.ALL;	--For real, ceil and log2 operators

LIBRARY work;
use work.leftimagePack.ALL;		--Get access to image height/width and pixelbitWidth

PACKAGE allconstantsPack IS

	CONSTANT D : integer := 160;
	CONSTANT DinBits : integer := integer(ceil(log2(real(D + 1))));
	
	--CONSTANT IMAGEWIDTH : integer := leftimageWidth;	--from leftimagePack
	CONSTANT IMAGEWIDTH : integer := 1226;	--from leftimagePack
	CONSTANT ImageWidthInBits : integer := integer(ceil(log2(real(IMAGEWIDTH + 1))));
	
	--CONSTANT IMAGEHEIGHT : integer := leftimageHeight;	--from leftimagePack
	CONSTANT IMAGEHEIGHT : integer := 375;	--from leftimagePack
	CONSTANT ImageHeightInBits : integer := integer(ceil(log2(real(IMAGEHEIGHT + 1))));
	
	CONSTANT PixelBitWidth : integer := leftimageMaxValueInBit;	--from leftimagePack. Raw Image Pixels have this bit width
	TYPE pixelBitWidthpipeLenDType IS ARRAY(NATURAL RANGE 1 TO D) OF unsigned(PixelBitWidth - 1 downto 0);
	
	--Used in find(min|max)(tree|reg) for initialization of first index vector:
	FUNCTION indices(constant LEN : natural) return pixelBitWidthpipeLenDType;
	
	CONSTANT EXPANDLEN : integer := 20;
	TYPE expanddispType IS ARRAY(NATURAL RANGE 1 TO EXPANDLEN) OF unsigned(DinBits - 1 downto 0);	--5 from left, 5 from current, 5 from right, 5 from previous 
	
	CONSTANT WindowLength : integer := 9;	--CT window size
	CONSTANT WindowHeight : integer := 9;
	CONSTANT WindowCenterY : integer := integer(floor(real(WindowHeight/2)));
	CONSTANT WindowCenterX : integer := integer(floor(real(WindowLength/2)));
		
	CONSTANT NumberOfCensusValues : integer := (WindowLength * WindowHeight) - 1;
	CONSTANT HammingBitSize : integer := integer(ceil(log2(real(NumberOfCensusValues + 1))));
	
	TYPE hammingvectorType IS ARRAY(natural range 1 TO EXPANDLEN) OF unsigned(HammingBitSize - 1 downto 0);
	
END allconstantsPack;

PACKAGE BODY allconstantsPack IS
	
	--Used in find(min|max)(tree|reg) for initialization of first index vector. 
	FUNCTION indices(constant LEN : natural) RETURN pixelBitWidthpipeLenDType IS
		VARIABLE tmp : pixelBitWidthpipeLenDType := (OTHERS => (OTHERS => '0'));
	BEGIN
		FOR i IN 1 TO LEN LOOP
			tmp(i) := to_unsigned(i, PixelBitWidth);
		END LOOP;
		
		RETURN tmp;
	
	END indices;

END allconstantsPack;

