--Libraries
library ieee;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library work;
use work.allconstantsPack.ALL;

--XOR-stage (Matlab generated) and optimal adder tree

ENTITY censusselective IS
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		leftin : IN unsigned(PixelBitWidth - 1 downto 0);
		rightin : IN unsigned(PixelBitWidth - 1 downto 0);
		expandin : IN expanddispType;		--expanded disparities, 1 TO 20
		hammingvectorout : OUT hammingvectorType	--hamming vector
	);
END censusselective;

ARCHITECTURE Behavioral OF censusselective IS

	COMPONENT censuspipefiforam 
		generic(
			IMAGEWIDTH : integer := ImageWidth;		--The census transform clockes in 2xIMAGEWIDTH + WINDOWLENGTH pixels before trasnform
			DATAWIDTH : integer := 8
		);
		PORT(
			clk : in std_logic;
			reset : in std_logic;
			datain : in unsigned(7 downto 0);
			--dataout : out unsigned(7 downto 0);		--Image pixel data drop out
			valid : out std_logic;
			targetrow : out unsigned(ImageHeightInBits - 1 downto 0);
			targetcol : out unsigned(ImageWidthInBits - 1 downto 0);
			censusout : out unsigned(NumberOfCensusValues - 1 downto 0)
		);
	END COMPONENT;

	--signal dataoutLsig, dataoutRsig : unsigned(7 downto 0);
	signal validRsig, validLsig : std_logic;
	signal censusLsig, censusRsig : unsigned(NumberOfCensusValues - 1 downto 0);
	
	TYPE censusvectorType IS ARRAY(natural range 1 TO D) OF unsigned(NumberOfCensusValues - 1 downto 0);	--For 9x9 CT, NumberOfCensusValues is 80!
	signal censusRpipe, censusRpipe_next : censusvectorType;
	
	signal colc, colc_next : unsigned(ImageWidthInBits - 1 downto 0);
	signal rowc, rowc_next : unsigned(ImageHeightInBits - 1 downto 0);
	
	signal targetrowL, targetrowR : unsigned(ImageHeightInBits - 1 downto 0);
	signal targetcolL, targetcolR : unsigned(ImageWidthInBits - 1 downto 0);
	
		--type hammingvectorType is ARRAY(1 TO EXPANDLEN) OF unsigned(NumberOfCensusValues - 1 downto 0);
	signal hammingvector, hammingvector_next : hammingvectorType;
	
BEGIN

	censusL : censuspipefiforam 
	PORT MAP(
		clk => clk,
		reset => reset,
		datain => leftin,
		valid => validLsig,
		targetrow => targetrowL,
		targetcol => targetcolL,
		censusout => censusLsig
	);
	
	censusR : censuspipefiforam 
	PORT MAP(
		clk => clk,
		reset => reset,
		datain => rightin,
		valid => validRsig,
		targetrow => targetrowR,
		targetcol => targetcolR,
		censusout => censusRsig
	);
	
	--Clocked process
	clkProc : PROCESS(clk, reset)
	begin
		IF reset = '1' then
			censusRpipe <= (OTHERS => (OTHERS => '0'));
			colc <= to_unsigned(1,ImageWidthInBits);
			rowc <= to_unsigned(1,ImageHeightInBits);	
			hammingvector <= (OTHERS => (OTHERS => '0'));
		ELSIF (clk'event AND clk = '1') THEN
			censusRpipe <= censusRpipe_next;
			colc <= colc_next;
			rowc <= rowc_next;
			hammingvector <= hammingvector_next;
		END IF;
	END PROCESS clkProc;

	censusPipeProc : PROCESS(censusRsig, censusRpipe)
	BEGIN
		censusRpipe_next(D) <= censusRsig;
		censusRpipe_next(1 TO D-1) <= censusRpipe(2 TO D);
	END PROCESS censusPipeProc;
	
	counterProc : PROCESS(colc, rowc) 
	BEGIN
		IF colc < IMAGEWIDTH THEN
			colc_next <= colc + 1;
			rowc_next <= rowc;
		ELSE
			colc_next <= to_unsigned(1,ImageWidthInBits);
			IF rowc < IMAGEHEIGHT THEN
				rowc_next <= rowc + 1;
			ELSE 
				rowc_next <= to_unsigned(1,ImageHeightInBits);
			END IF;
		END IF;
	END PROCESS counterProc;
	
	
	censusProc2 : PROCESS(censusLsig, censusRpipe, expandin, hammingvector)
		type resvectorType1 IS ARRAY(NATURAL RANGE 1 TO 40) OF unsigned(1 DOWNTO 0);
		type resvectorType2 IS ARRAY(NATURAL RANGE 1 TO 20) OF unsigned(2 DOWNTO 0);
		type resvectorType3 IS ARRAY(NATURAL RANGE 1 TO 10) OF unsigned(3 DOWNTO 0);
		type resvectorType4 IS ARRAY(NATURAL RANGE 1 TO 5) OF unsigned(4 DOWNTO 0);
		type resvectorType5 IS ARRAY(NATURAL RANGE 1 TO 3) OF unsigned(5 DOWNTO 0);
		type resvectorType6 IS ARRAY(NATURAL RANGE 1 TO 2) OF unsigned(6 DOWNTO 0);
	
		variable final : unsigned(HammingBitSize - 1 downto 0);		--HammingBitSize 
		variable res1 : resvectorType1;
		variable res2 : resvectorType2;
		variable res3 : resvectorType3;
		variable res4 : resvectorType4;
		variable res5 : resvectorType5;
		variable res6 : resvectorType6;
		variable tmp1, tmp2 : std_logic;
		variable valuetocheck : unsigned(NumberOfCensusValues - 1 downto 0) := (OTHERS => '0');
	
	BEGIN
		hammingvector_next <= hammingvector;
		
		FOR j IN 1 TO EXPANDLEN LOOP
			final := (OTHERS => '0');
			IF expandin(j) > 0 THEN
				valuetocheck := censusRpipe(to_integer(expandin(j)));		--Variable is possibly not required

				tmp1 := censusLsig(0) XOR valuetocheck(0); tmp2 := censusLsig(1) XOR valuetocheck(1);
				IF tmp1 = tmp2 THEN res1(1) := (tmp1 OR tmp2) & '0'; ELSE res1(1) := "01"; END IF;

				tmp1 := censusLsig(2) XOR valuetocheck(2);tmp2 := censusLsig(3) XOR valuetocheck(3);
				IF tmp1 = tmp2 THEN res1(2) := (tmp1 OR tmp2) & '0'; ELSE res1(2) := "01"; END IF;

				tmp1 := censusLsig(4) XOR valuetocheck(4);tmp2 := censusLsig(5) XOR valuetocheck(5);
				IF tmp1 = tmp2 THEN res1(3) := (tmp1 OR tmp2) & '0'; ELSE res1(3) := "01"; END IF;

				tmp1 := censusLsig(6) XOR valuetocheck(6);tmp2 := censusLsig(7) XOR valuetocheck(7);
				IF tmp1 = tmp2 THEN res1(4) := (tmp1 OR tmp2) & '0'; ELSE res1(4) := "01"; END IF;

				tmp1 := censusLsig(8) XOR valuetocheck(8);tmp2 := censusLsig(9) XOR valuetocheck(9);
				IF tmp1 = tmp2 THEN res1(5) := (tmp1 OR tmp2) & '0'; ELSE res1(5) := "01"; END IF;

				tmp1 := censusLsig(10) XOR valuetocheck(10);tmp2 := censusLsig(11) XOR valuetocheck(11);
				IF tmp1 = tmp2 THEN res1(6) := (tmp1 OR tmp2) & '0'; ELSE res1(6) := "01"; END IF;

				tmp1 := censusLsig(12) XOR valuetocheck(12);tmp2 := censusLsig(13) XOR valuetocheck(13);
				IF tmp1 = tmp2 THEN res1(7) := (tmp1 OR tmp2) & '0'; ELSE res1(7) := "01"; END IF;

				tmp1 := censusLsig(14) XOR valuetocheck(14);tmp2 := censusLsig(15) XOR valuetocheck(15);
				IF tmp1 = tmp2 THEN res1(8) := (tmp1 OR tmp2) & '0'; ELSE res1(8) := "01"; END IF;

				tmp1 := censusLsig(16) XOR valuetocheck(16);tmp2 := censusLsig(17) XOR valuetocheck(17);
				IF tmp1 = tmp2 THEN res1(9) := (tmp1 OR tmp2) & '0'; ELSE res1(9) := "01"; END IF;

				tmp1 := censusLsig(18) XOR valuetocheck(18);tmp2 := censusLsig(19) XOR valuetocheck(19);
				IF tmp1 = tmp2 THEN res1(10) := (tmp1 OR tmp2) & '0'; ELSE res1(10) := "01"; END IF;

				tmp1 := censusLsig(20) XOR valuetocheck(20);tmp2 := censusLsig(21) XOR valuetocheck(21);
				IF tmp1 = tmp2 THEN res1(11) := (tmp1 OR tmp2) & '0'; ELSE res1(11) := "01"; END IF;

				tmp1 := censusLsig(22) XOR valuetocheck(22);tmp2 := censusLsig(23) XOR valuetocheck(23);
				IF tmp1 = tmp2 THEN res1(12) := (tmp1 OR tmp2) & '0'; ELSE res1(12) := "01"; END IF;

				tmp1 := censusLsig(24) XOR valuetocheck(24);tmp2 := censusLsig(25) XOR valuetocheck(25);
				IF tmp1 = tmp2 THEN res1(13) := (tmp1 OR tmp2) & '0'; ELSE res1(13) := "01"; END IF;

				tmp1 := censusLsig(26) XOR valuetocheck(26);tmp2 := censusLsig(27) XOR valuetocheck(27);
				IF tmp1 = tmp2 THEN res1(14) := (tmp1 OR tmp2) & '0'; ELSE res1(14) := "01"; END IF;

				tmp1 := censusLsig(28) XOR valuetocheck(28);tmp2 := censusLsig(29) XOR valuetocheck(29);
				IF tmp1 = tmp2 THEN res1(15) := (tmp1 OR tmp2) & '0'; ELSE res1(15) := "01"; END IF;

				tmp1 := censusLsig(30) XOR valuetocheck(30);tmp2 := censusLsig(31) XOR valuetocheck(31);
				IF tmp1 = tmp2 THEN res1(16) := (tmp1 OR tmp2) & '0'; ELSE res1(16) := "01"; END IF;

				tmp1 := censusLsig(32) XOR valuetocheck(32);tmp2 := censusLsig(33) XOR valuetocheck(33);
				IF tmp1 = tmp2 THEN res1(17) := (tmp1 OR tmp2) & '0'; ELSE res1(17) := "01"; END IF;

				tmp1 := censusLsig(34) XOR valuetocheck(34);tmp2 := censusLsig(35) XOR valuetocheck(35);
				IF tmp1 = tmp2 THEN res1(18) := (tmp1 OR tmp2) & '0'; ELSE res1(18) := "01"; END IF;

				tmp1 := censusLsig(36) XOR valuetocheck(36);tmp2 := censusLsig(37) XOR valuetocheck(37);
				IF tmp1 = tmp2 THEN res1(19) := (tmp1 OR tmp2) & '0'; ELSE res1(19) := "01"; END IF;

				tmp1 := censusLsig(38) XOR valuetocheck(38);tmp2 := censusLsig(39) XOR valuetocheck(39);
				IF tmp1 = tmp2 THEN res1(20) := (tmp1 OR tmp2) & '0'; ELSE res1(20) := "01"; END IF;

				tmp1 := censusLsig(40) XOR valuetocheck(40);tmp2 := censusLsig(41) XOR valuetocheck(41);
				IF tmp1 = tmp2 THEN res1(21) := (tmp1 OR tmp2) & '0'; ELSE res1(21) := "01"; END IF;

				tmp1 := censusLsig(42) XOR valuetocheck(42);tmp2 := censusLsig(43) XOR valuetocheck(43);
				IF tmp1 = tmp2 THEN res1(22) := (tmp1 OR tmp2) & '0'; ELSE res1(22) := "01"; END IF;

				tmp1 := censusLsig(44) XOR valuetocheck(44);tmp2 := censusLsig(45) XOR valuetocheck(45);
				IF tmp1 = tmp2 THEN res1(23) := (tmp1 OR tmp2) & '0'; ELSE res1(23) := "01"; END IF;

				tmp1 := censusLsig(46) XOR valuetocheck(46);tmp2 := censusLsig(47) XOR valuetocheck(47);
				IF tmp1 = tmp2 THEN res1(24) := (tmp1 OR tmp2) & '0'; ELSE res1(24) := "01"; END IF;

				tmp1 := censusLsig(48) XOR valuetocheck(48);tmp2 := censusLsig(49) XOR valuetocheck(49);
				IF tmp1 = tmp2 THEN res1(25) := (tmp1 OR tmp2) & '0'; ELSE res1(25) := "01"; END IF;

				tmp1 := censusLsig(50) XOR valuetocheck(50);tmp2 := censusLsig(51) XOR valuetocheck(51);
				IF tmp1 = tmp2 THEN res1(26) := (tmp1 OR tmp2) & '0'; ELSE res1(26) := "01"; END IF;

				tmp1 := censusLsig(52) XOR valuetocheck(52);tmp2 := censusLsig(53) XOR valuetocheck(53);
				IF tmp1 = tmp2 THEN res1(27) := (tmp1 OR tmp2) & '0'; ELSE res1(27) := "01"; END IF;

				tmp1 := censusLsig(54) XOR valuetocheck(54);tmp2 := censusLsig(55) XOR valuetocheck(55);
				IF tmp1 = tmp2 THEN res1(28) := (tmp1 OR tmp2) & '0'; ELSE res1(28) := "01"; END IF;

				tmp1 := censusLsig(56) XOR valuetocheck(56);tmp2 := censusLsig(57) XOR valuetocheck(57);
				IF tmp1 = tmp2 THEN res1(29) := (tmp1 OR tmp2) & '0'; ELSE res1(29) := "01"; END IF;

				tmp1 := censusLsig(58) XOR valuetocheck(58);tmp2 := censusLsig(59) XOR valuetocheck(59);
				IF tmp1 = tmp2 THEN res1(30) := (tmp1 OR tmp2) & '0'; ELSE res1(30) := "01"; END IF;

				tmp1 := censusLsig(60) XOR valuetocheck(60);tmp2 := censusLsig(61) XOR valuetocheck(61);
				IF tmp1 = tmp2 THEN res1(31) := (tmp1 OR tmp2) & '0'; ELSE res1(31) := "01"; END IF;

				tmp1 := censusLsig(62) XOR valuetocheck(62);tmp2 := censusLsig(63) XOR valuetocheck(63);
				IF tmp1 = tmp2 THEN res1(32) := (tmp1 OR tmp2) & '0'; ELSE res1(32) := "01"; END IF;

				tmp1 := censusLsig(64) XOR valuetocheck(64);tmp2 := censusLsig(65) XOR valuetocheck(65);
				IF tmp1 = tmp2 THEN res1(33) := (tmp1 OR tmp2) & '0'; ELSE res1(33) := "01"; END IF;

				tmp1 := censusLsig(66) XOR valuetocheck(66);tmp2 := censusLsig(67) XOR valuetocheck(67);
				IF tmp1 = tmp2 THEN res1(34) := (tmp1 OR tmp2) & '0'; ELSE res1(34) := "01"; END IF;

				tmp1 := censusLsig(68) XOR valuetocheck(68);tmp2 := censusLsig(69) XOR valuetocheck(69);
				IF tmp1 = tmp2 THEN res1(35) := (tmp1 OR tmp2) & '0'; ELSE res1(35) := "01"; END IF;

				tmp1 := censusLsig(70) XOR valuetocheck(70);tmp2 := censusLsig(71) XOR valuetocheck(71);
				IF tmp1 = tmp2 THEN res1(36) := (tmp1 OR tmp2) & '0'; ELSE res1(36) := "01"; END IF;

				tmp1 := censusLsig(72) XOR valuetocheck(72);tmp2 := censusLsig(73) XOR valuetocheck(73);
				IF tmp1 = tmp2 THEN res1(37) := (tmp1 OR tmp2) & '0'; ELSE res1(37) := "01"; END IF;

				tmp1 := censusLsig(74) XOR valuetocheck(74);tmp2 := censusLsig(75) XOR valuetocheck(75);
				IF tmp1 = tmp2 THEN res1(38) := (tmp1 OR tmp2) & '0'; ELSE res1(38) := "01"; END IF;

				tmp1 := censusLsig(76) XOR valuetocheck(76);tmp2 := censusLsig(77) XOR valuetocheck(77);
				IF tmp1 = tmp2 THEN res1(39) := (tmp1 OR tmp2) & '0'; ELSE res1(39) := "01"; END IF;

				tmp1 := censusLsig(78) XOR valuetocheck(78);tmp2 := censusLsig(79) XOR valuetocheck(79);
				IF tmp1 = tmp2 THEN res1(40) := (tmp1 OR tmp2) & '0'; ELSE res1(40) := "01"; END IF;
				--XOR-section done

				--Sum section begins:
				--Stage 2:
				res2(1) := ('0' & res1(1)) + ('0' & res1(2));
				res2(2) := ('0' & res1(3)) + ('0' & res1(4));
				res2(3) := ('0' & res1(5)) + ('0' & res1(6));
				res2(4) := ('0' & res1(7)) + ('0' & res1(8));
				res2(5) := ('0' & res1(9)) + ('0' & res1(10));
				res2(6) := ('0' & res1(11)) + ('0' & res1(12));
				res2(7) := ('0' & res1(13)) + ('0' & res1(14));
				res2(8) := ('0' & res1(15)) + ('0' & res1(16));
				res2(9) := ('0' & res1(17)) + ('0' & res1(18));
				res2(10) := ('0' & res1(19)) + ('0' & res1(20));
				res2(11) := ('0' & res1(21)) + ('0' & res1(22));
				res2(12) := ('0' & res1(23)) + ('0' & res1(24));
				res2(13) := ('0' & res1(25)) + ('0' & res1(26));
				res2(14) := ('0' & res1(27)) + ('0' & res1(28));
				res2(15) := ('0' & res1(29)) + ('0' & res1(30));
				res2(16) := ('0' & res1(31)) + ('0' & res1(32));
				res2(17) := ('0' & res1(33)) + ('0' & res1(34));
				res2(18) := ('0' & res1(35)) + ('0' & res1(36));
				res2(19) := ('0' & res1(37)) + ('0' & res1(38));
				res2(20) := ('0' & res1(39)) + ('0' & res1(40));

				--Stage 3:
				res3(1) := ('0' & res2(1)) + ('0' & res2(2));
				res3(2) := ('0' & res2(3)) + ('0' & res2(4));
				res3(3) := ('0' & res2(5)) + ('0' & res2(6));
				res3(4) := ('0' & res2(7)) + ('0' & res2(8));
				res3(5) := ('0' & res2(9)) + ('0' & res2(10));
				res3(6) := ('0' & res2(11)) + ('0' & res2(12));
				res3(7) := ('0' & res2(13)) + ('0' & res2(14));
				res3(8) := ('0' & res2(15)) + ('0' & res2(16));
				res3(9) := ('0' & res2(17)) + ('0' & res2(18));
				res3(10) := ('0' & res2(19)) + ('0' & res2(20));

				--Stage 4:
				res4(1) := ('0' & res3(1)) + ('0' & res3(2));
				res4(2) := ('0' & res3(3)) + ('0' & res3(4));
				res4(3) := ('0' & res3(5)) + ('0' & res3(6));
				res4(4) := ('0' & res3(7)) + ('0' & res3(8));
				res4(5) := ('0' & res3(9)) + ('0' & res3(10));

				--Stage 5:
				res5(1) := ('0' & res4(1)) + ('0' & res4(2));
				res5(2) := ('0' & res4(3)) + ('0' & res4(4));
				res5(3) := ('0' & res4(5));

				--Stage 6:
				res6(1) := ('0' & res5(1)) + ('0' & res5(2));
				res6(2) := ('0' & res5(3));

				--Final stage
				final := res6(1) + res6(2);	--Zusätzliche führende null nicht benötigt, da das Ergebnis definitiv passt!
			ELSE
				final := (OTHERS => '0');
			END IF;
			
			hammingvector_next(j) <= final;
		END LOOP;
	END PROCESS censusProc2;
	
	
	--censusRout <= censusRpipe;
	--censusLout <= censusLsig;
	
	hammingvectorout <= hammingvector;


 end Behavioral;
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
