--Libraries
library ieee;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library work;
use work.allconstantsPack.ALL;

--Using two CASE-Statements, one for XOR stage, one for the first adder stage
ENTITY censusselective IS
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		leftin : IN unsigned(PixelBitWidth - 1 downto 0);
		rightin : IN unsigned(PixelBitWidth - 1 downto 0);
		expandin : IN expanddispType;		--expanded disparities, 1 TO 20
		hammingvectorout : OUT hammingvectorType	--hamming vector
	);
END censusselective;

ARCHITECTURE Behavioral OF censusselective IS

	COMPONENT censuspipefiforam 
		generic(
			IMAGEWIDTH : integer := ImageWidth;		--The census transform clockes in 2xIMAGEWIDTH + WINDOWLENGTH pixels before trasnform
			DATAWIDTH : integer := 8
		);
		PORT(
			clk : in std_logic;
			reset : in std_logic;
			datain : in unsigned(7 downto 0);
			--dataout : out unsigned(7 downto 0);		--Image pixel data drop out
			valid : out std_logic;
			targetrow : out unsigned(ImageHeightInBits - 1 downto 0);
			targetcol : out unsigned(ImageWidthInBits - 1 downto 0);
			censusout : out unsigned(NumberOfCensusValues - 1 downto 0)
		);
	END COMPONENT;

	--signal dataoutLsig, dataoutRsig : unsigned(7 downto 0);
	signal validRsig, validLsig : std_logic;
	signal censusLsig, censusRsig : unsigned(NumberOfCensusValues - 1 downto 0);
	
	TYPE censusvectorType IS ARRAY(natural range 1 TO D) OF unsigned(NumberOfCensusValues - 1 downto 0);	--For 9x9 CT, NumberOfCensusValues is 80!
	signal censusRpipe, censusRpipe_next : censusvectorType;
	
	signal colc, colc_next : unsigned(ImageWidthInBits - 1 downto 0);
	signal rowc, rowc_next : unsigned(ImageHeightInBits - 1 downto 0);
	
	signal targetrowL, targetrowR : unsigned(ImageHeightInBits - 1 downto 0);
	signal targetcolL, targetcolR : unsigned(ImageWidthInBits - 1 downto 0);
	
		--type hammingvectorType is ARRAY(1 TO EXPANDLEN) OF unsigned(NumberOfCensusValues - 1 downto 0);
	signal hammingvector, hammingvector_next : hammingvectorType;
	
BEGIN

	censusL : censuspipefiforam 
	PORT MAP(
		clk => clk,
		reset => reset,
		datain => leftin,
		valid => validLsig,
		targetrow => targetrowL,
		targetcol => targetcolL,
		censusout => censusLsig
	);
	
	censusR : censuspipefiforam 
	PORT MAP(
		clk => clk,
		reset => reset,
		datain => rightin,
		valid => validRsig,
		targetrow => targetrowR,
		targetcol => targetcolR,
		censusout => censusRsig
	);
	
	--Clocked process
	clkProc : PROCESS(clk, reset)
	begin
		IF reset = '1' then
			censusRpipe <= (OTHERS => (OTHERS => '0'));
			colc <= to_unsigned(1,ImageWidthInBits);
			rowc <= to_unsigned(1,ImageHeightInBits);	
			hammingvector <= (OTHERS => (OTHERS => '0'));
		ELSIF (clk'event AND clk = '1') THEN
			censusRpipe <= censusRpipe_next;
			colc <= colc_next;
			rowc <= rowc_next;
			hammingvector <= hammingvector_next;
		END IF;
	END PROCESS clkProc;

	censusPipeProc : PROCESS(censusRsig, censusRpipe)
	BEGIN
		censusRpipe_next(D) <= censusRsig;
		censusRpipe_next(1 TO D-1) <= censusRpipe(2 TO D);
	END PROCESS censusPipeProc;
	
	counterProc : PROCESS(colc, rowc) 
	BEGIN
		IF colc < IMAGEWIDTH THEN
			colc_next <= colc + 1;
			rowc_next <= rowc;
		ELSE
			colc_next <= to_unsigned(1,ImageWidthInBits);
			IF rowc < IMAGEHEIGHT THEN
				rowc_next <= rowc + 1;
			ELSE 
				rowc_next <= to_unsigned(1,ImageHeightInBits);
			END IF;
		END IF;
	END PROCESS counterProc;
	
	
	censusProc2 : PROCESS(censusLsig, censusRpipe, expandin, hammingvector)
		type resvectorType1 IS ARRAY(NATURAL RANGE 1 TO 40) OF unsigned(1 DOWNTO 0);
		type resvectorType2 IS ARRAY(NATURAL RANGE 1 TO 20) OF unsigned(2 DOWNTO 0);
		type resvectorType3 IS ARRAY(NATURAL RANGE 1 TO 10) OF unsigned(3 DOWNTO 0);
		type resvectorType4 IS ARRAY(NATURAL RANGE 1 TO 5) OF unsigned(4 DOWNTO 0);
		type resvectorType5 IS ARRAY(NATURAL RANGE 1 TO 3) OF unsigned(5 DOWNTO 0);
		type resvectorType6 IS ARRAY(NATURAL RANGE 1 TO 2) OF unsigned(6 DOWNTO 0);
	
		variable final : unsigned(HammingBitSize - 1 downto 0);		--HammingBitSize 
		variable res1 : resvectorType1;
		variable res2 : resvectorType2;
		variable res3 : resvectorType3;
		variable res4 : resvectorType4;
		variable res5 : resvectorType5;
		variable res6 : resvectorType6;
		--variable tmp1, tmp2 : std_logic;
		variable tmp : unsigned(3 downto 0) := "0000";
		variable valuetocheck : unsigned(NumberOfCensusValues - 1 downto 0) := (OTHERS => '0');
	
	BEGIN
		hammingvector_next <= hammingvector;
		
		FOR j IN 1 TO EXPANDLEN LOOP
			final := (OTHERS => '0');
			IF expandin(j) > 0 THEN
				valuetocheck := censusRpipe(to_integer(expandin(j)));		--Variable is possibly not required

				FOR i IN 0 TO 39 LOOP
					tmp := valuetocheck(i*2) & censusLsig(i*2) & valuetocheck((i*2)+1) & censusLsig((i*2)+1);	--Concatenate
					
					CASE tmp IS
						WHEN "0000" => res1(i+1) := "00";
						WHEN "0001" => res1(i+1) := "01";
						WHEN "0010" => res1(i+1) := "01";
						WHEN "0011" => res1(i+1) := "00";
						
						WHEN "0100" => res1(i+1) := "01";
						WHEN "0101" => res1(i+1) := "10";
						WHEN "0110" => res1(i+1) := "10";
						WHEN "0111" => res1(i+1) := "01";
						
						WHEN "1000" => res1(i+1) := "01";
						WHEN "1001" => res1(i+1) := "10";
						WHEN "1010" => res1(i+1) := "10";
						WHEN "1011" => res1(i+1) := "01";
						
						WHEN "1100" => res1(i+1) := "00";
						WHEN "1101" => res1(i+1) := "01";
						WHEN "1110" => res1(i+1) := "01";
						WHEN "1111" => res1(i+1) := "00";
						
						WHEN OTHERS => res1(i+1) := "00";
					END CASE;
				END LOOP;
				--XOR-section done

				FOR i IN 1 TO 20 LOOP
					tmp := res1((i*2)-1) & res1(i*2);
					
					CASE tmp IS
						WHEN "0000" => res2(i) := "000";
						WHEN "0001" => res2(i) := "001";
						WHEN "0010" => res2(i) := "010";
						WHEN "0011" => res2(i) := "011";
						
						WHEN "0100" => res2(i) := "001";
						WHEN "0101" => res2(i) := "010";
						WHEN "0110" => res2(i) := "011";
						WHEN "0111" => res2(i) := "100";
						
						WHEN "1000" => res2(i) := "010";
						WHEN "1001" => res2(i) := "011";
						WHEN "1010" => res2(i) := "100";
						WHEN "1011" => res2(i) := "101";
						
						WHEN "1100" => res2(i) := "011";
						WHEN "1101" => res2(i) := "100";
						WHEN "1110" => res2(i) := "101";
						WHEN "1111" => res2(i) := "110";
						
						WHEN OTHERS => res2(i) := "000";
					END CASE;
				END LOOP;
				
				
				--Stage 3:
				res3(1) := ('0' & res2(1)) + ('0' & res2(2));
				res3(2) := ('0' & res2(3)) + ('0' & res2(4));
				res3(3) := ('0' & res2(5)) + ('0' & res2(6));
				res3(4) := ('0' & res2(7)) + ('0' & res2(8));
				res3(5) := ('0' & res2(9)) + ('0' & res2(10));
				res3(6) := ('0' & res2(11)) + ('0' & res2(12));
				res3(7) := ('0' & res2(13)) + ('0' & res2(14));
				res3(8) := ('0' & res2(15)) + ('0' & res2(16));
				res3(9) := ('0' & res2(17)) + ('0' & res2(18));
				res3(10) := ('0' & res2(19)) + ('0' & res2(20));

				--Stage 4:
				res4(1) := ('0' & res3(1)) + ('0' & res3(2));
				res4(2) := ('0' & res3(3)) + ('0' & res3(4));
				res4(3) := ('0' & res3(5)) + ('0' & res3(6));
				res4(4) := ('0' & res3(7)) + ('0' & res3(8));
				res4(5) := ('0' & res3(9)) + ('0' & res3(10));

				--Stage 5:
				res5(1) := ('0' & res4(1)) + ('0' & res4(2));
				res5(2) := ('0' & res4(3)) + ('0' & res4(4));
				res5(3) := ('0' & res4(5));

				--Stage 6:
				res6(1) := ('0' & res5(1)) + ('0' & res5(2));
				res6(2) := ('0' & res5(3));

				--Final stage
				final := res6(1) + res6(2);	--Zustzliche fhrende null nicht bentigt, da das Ergebnis definitiv passt!
			ELSE
				final := (OTHERS => '0');
			END IF;
			
			hammingvector_next(j) <= final;
		END LOOP;
	END PROCESS censusProc2;
	
	
	--censusRout <= censusRpipe;
	--censusLout <= censusLsig;
	
	hammingvectorout <= hammingvector;


 end Behavioral;
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
