--------------------------------------------------------------------------------
library ieee;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library work;
use work.allconstantsPack.ALL;
use work.leftimagePack.ALL;
use work.rightimagePack.ALL;
use work.referencedataPack.ALL;

library std;
use std.textio.all;

 
ENTITY tb_censusselective IS
END tb_censusselective;
 
ARCHITECTURE behavior OF tb_censusselective IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT censusselective
    PORT(
         clk : IN  std_logic;
         reset : IN  std_logic;
         leftin : IN  unsigned(7 downto 0);
         rightin : IN  unsigned(7 downto 0);
         expandin : IN  expanddispType;
         hammingvectorout : OUT hammingvectorType
        );
    END COMPONENT;
    

   --Inputs
   signal clk : std_logic := '0';
   signal reset : std_logic := '0';
   signal leftin : unsigned(7 downto 0) := (others => '0');
   signal rightin : unsigned(7 downto 0) := (others => '0');
   signal expandin : expanddispType := (others => (others => '0'));

 	--Outputs
   signal hammingvectorout : hammingvectorType;

   -- Clock period definitions
   constant clk_period : time := 10 ns;
	
	file fileref: TEXT open write_mode is "censusProc2.txt";
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: censusselective PORT MAP (
          clk => clk,
          reset => reset,
          leftin => leftin,
          rightin => rightin,
          expandin => expandin,
          hammingvectorout => hammingvectorout
        );

   -- Clock process definitions
   clk_process : process
   begin
		clk <= '0';
		wait for clk_period/2;
		clk <= '1';
		wait for clk_period/2;
   end process;
 
	 
	stimProc : PROCESS
	BEGIN
		reset <= '1';
		wait for clk_period * 2;
		reset <= '0';
		
		FOR y IN 0 TO 10 LOOP
			FOR x in 0 TO ImageWidth - 1 LOOP
				leftin <= leftimage(y,x);
				rightin <= rightimage(y,x);
				
				FOR i IN 1 TO EXPANDLEN LOOP
					expandin(i) <= "00" & referencedata(y,(x * EXPANDLEN)+i);
				END LOOP;
				
				wait for clk_period;
			END LOOP;
		END LOOP;
			
		wait for clk_period * 10;
		
		report "Fertsch. Alled jut" severity failure;
		
	END PROCESS;
	
	recordProc : PROCESS
	variable writevar : line;

	BEGIN
		wait for clk_period * 2;
		
		FOR y IN 0 TO 10 LOOP
			FOR x in 0 TO ImageWidth - 1 LOOP
				
				FOR i IN 1 TO EXPANDLEN LOOP
					
					write(writevar, INTEGER'image(to_integer(hammingvectorout(i))) & " ");	--Fill variable
					
				END LOOP;
				writeline(fileref, writevar);	--Write variable to file
				write(writevar, string'(""));		--flush variable
				wait for clk_period;
			END LOOP;
		END LOOP;
		
	
	
	END PROCESS;
	
END;
