--Expands disparity values. 
-- Receives indexin from previous houghline module. indexin changes every D cycles. 
-- Checks range of indexin and expands it by +/- 2 if possible. 
-- Uses three small pipes: nplus1, n and nminus1. The expanded indexin is clocked into nplus1. After D cycles, nplus1 is 
-- shifted to n and n to nminus1
-- REceives also previousdisp which stands for the found disparity of the previous row for each pixel. It changes at each clock cycle
-- if colc is bigger than D. previousdisp is also expanded by +/-2 if possible. 
-- As result, an index vector with 20 index values to check is clocked out at each clock cycle. The indices are unordered 
-- and double entries are possible. Zeros are unvalid.

--schfra 15.4.2014

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.allconstantsPack.ALL;	--contains windowlength and windowheight. From these values, the required Fifos are instantiated, the proper fifotopipe signals, pipetofifo, census length etc.

ENTITY expandisparity is
	port(
		clk : in std_logic;
		reset : in std_logic;
		colc : in unsigned(ImageWidthInBits - 1 downto 0);	--column counter
		indexin : in unsigned(7 downto 0);		--from houghline module
		prevdispin : in unsigned(7 downto 0);	--disparity from previous row  	
		expandout : out expanddispType			--vector containing 20 index values
	);
END expandisparity;

ARCHITECTURE behaviour OF expandisparity IS

signal nminus1, nminus1_next : indvecType;
signal n, n_next : indvecType;
signal nplus1, nplus1_next : indvecType;
signal prevdispexp, prevdispexp_next : indvecType;

signal internalcolc, internalcolc_next : unsigned(ImageWidthInBits - 1 downto 0);	--internal column counter runs from 1...160

BEGIN

clkProc : PROCESS(clk, reset)
BEGIN
	IF reset = '1' THEN
		nminus1 <= (OTHERS => (OTHERS => '0'));
		n <= (OTHERS => (OTHERS => '0'));
		nplus1 <= (OTHERS => (OTHERS => '0'));
		prevdispexp <= (OTHERS => (OTHERS => '0'));
		internalcolc <= to_unsigned(1,ImageWidthInBits);
	ELSIF clk'event AND clk = '1' THEN
		nminus1 <= nminus1_next;
		n <= n_next;
		nplus1 <= nplus1_next;
		prevdispexp <= prevdispexp_next;
		internalcolc <= internalcolc_next;
	END IF;
END PROCESS;

cmbProc2 : PROCESS(indexin, prevdispin)
-- variable smaller, bigger : unsigned(7 downto 0) := (OTHERS => '0');
-- variable absdiff : integer range 0 to 160 := 0;

BEGIN
	
	-- IF indexin >= 1 AND indexin <= D AND prevdispin >= 1 AND prevdispin <= D THEN
		
		-- smaller := indexin;
		-- bigger := prevdispin;
	
		-- IF indexin > prevdispin THEN
			-- smaller := prevdispin;
			-- bigger := indexin;
		-- END IF;
		
		-- absdiff := to_integer(bigger) - to_integer(smaller);
		
		-- CASE absdiff IS
			-- when 0 => 
					-- nplus1_next(3) <= indexin;
					-- IF indexin > 2   THEN nplus1_next(1) <= indexin - 2; ELSE nplus1_next(1) <= (OTHERS => '0'); END IF;
					-- IF indexin > 1   THEN nplus1_next(2) <= indexin - 1; ELSE nplus1_next(2) <= (OTHERS => '0'); END IF;
					-- IF indexin < D-1 THEN nplus1_next(5) <= indexin + 2; ELSE nplus1_next(5) <= (OTHERS => '0'); END IF;
					-- IF indexin < D   THEN nplus1_next(4) <= indexin + 1; ELSE nplus1_next(4) <= (OTHERS => '0'); END IF;
					
					-- --prevdispexp_next(3) <= prevdispin;
					-- --IF prevdispin > 2   THEN prevdispexp_next(1) <= prevdispin - 2; ELSE prevdispexp_next(1) <= (OTHERS => '0'); END IF;
					-- --IF prevdispin > 1   THEN prevdispexp_next(2) <= prevdispin - 1; ELSE prevdispexp_next(2) <= (OTHERS => '0'); END IF;
					-- --IF prevdispin < D-1 THEN prevdispexp_next(5) <= prevdispin + 2; ELSE prevdispexp_next(5) <= (OTHERS => '0'); END IF;
					-- --IF prevdispin < D   THEN prevdispexp_next(4) <= prevdispin + 1; ELSE prevdispexp_next(4) <= (OTHERS => '0'); END IF;
					
			-- when 1 |  2 | 3 | 4 =>
				-- nplus1_next <= (OTHERS => (OTHERS => '0'));
				-- prevdispexp_next <= (OTHERS => (OTHERS => '0'));
			-- --when 2 => 
			-- --when 3 =>
			-- --when 4 =>
			-- when OTHERS => 
				-- nplus1_next(3) <= indexin;
				-- IF indexin > 2   THEN nplus1_next(1) <= indexin - 2; ELSE nplus1_next(1) <= (OTHERS => '0'); END IF;
				-- IF indexin > 1   THEN nplus1_next(2) <= indexin - 1; ELSE nplus1_next(2) <= (OTHERS => '0'); END IF;
				-- IF indexin < D-1 THEN nplus1_next(5) <= indexin + 2; ELSE nplus1_next(5) <= (OTHERS => '0'); END IF;
				-- IF indexin < D   THEN nplus1_next(4) <= indexin + 1; ELSE nplus1_next(4) <= (OTHERS => '0'); END IF;
				
				-- prevdispexp_next(3) <= prevdispin;
				-- IF prevdispin > 2   THEN prevdispexp_next(1) <= prevdispin - 2; ELSE prevdispexp_next(1) <= (OTHERS => '0'); END IF;
				-- IF prevdispin > 1   THEN prevdispexp_next(2) <= prevdispin - 1; ELSE prevdispexp_next(2) <= (OTHERS => '0'); END IF;
				-- IF prevdispin < D-1 THEN prevdispexp_next(5) <= prevdispin + 2; ELSE prevdispexp_next(5) <= (OTHERS => '0'); END IF;
				-- IF prevdispin < D   THEN prevdispexp_next(4) <= prevdispin + 1; ELSE prevdispexp_next(4) <= (OTHERS => '0'); END IF;
					
		-- END CASE;
		
	-- ELSE
		-- nplus1_next <= (OTHERS => (OTHERS => '0'));
		-- prevdispexp_next <= (OTHERS => (OTHERS => '0'));
	-- END IF;
			
	
	
	--Expand index in:
	IF indexin >= 1 AND indexin <= D THEN
		nplus1_next(3) <= indexin;
		
		IF indexin > 2 THEN
			nplus1_next(1) <= indexin - 2;
		ELSE
			nplus1_next(1) <= (OTHERS => '0');
		END IF;
	
		IF indexin > 1 THEN
			nplus1_next(2) <= indexin - 1;
		ELSE
			nplus1_next(2) <= (OTHERS => '0');
		END IF;
		
		IF indexin < D - 1 THEN
			nplus1_next(5) <= indexin + 2;
		ELSE
			nplus1_next(5) <= (OTHERS => '0');
		END IF;
		
		IF indexin < D THEN
			nplus1_next(4) <= indexin + 1;
		ELSE
			nplus1_next(4) <= (OTHERS => '0');
		END IF;
	ELSE
		nplus1_next <= (OTHERS => (OTHERS => '0'));
	END IF;
	
	--Expand previous disparity value
	IF prevdispin >= 1 AND prevdispin <= D THEN
		prevdispexp_next(3) <= prevdispin;
		
		IF prevdispin > 2 THEN
			prevdispexp_next(1) <= prevdispin - 2;
		ELSE
			prevdispexp_next(1) <= (OTHERS => '0');
		END IF;

		IF prevdispin > 1 THEN
			prevdispexp_next(2) <= prevdispin - 1;
		ELSE
			prevdispexp_next(2) <= (OTHERS => '0');
		END IF;
		
		IF prevdispin < D - 1 THEN
			prevdispexp_next(5) <= prevdispin + 2;
		ELSE
			prevdispexp_next(5) <= (OTHERS => '0');
		END IF;
		
		IF prevdispin < D THEN
			prevdispexp_next(4) <= prevdispin + 1;
		ELSE
			prevdispexp_next(4) <= (OTHERS => '0');
		END IF;
	ELSE
		prevdispexp_next <= (OTHERS => (OTHERS => '0'));
	END IF;
END PROCESS;

--After D cycles, shift nplus to n and n to nminus1:
shftProc : PROCESS(nminus1, n, nplus1, colc, internalcolc)
BEGIN
	n_next <= n;
	nminus1_next <= nminus1;
	
	IF colc > D AND internalcolc >= D THEN
		n_next <= nplus1;
		nminus1_next <= n;
	END IF;
END PROCESS;
		
--module for internal column counter which counts from 1 ... D
intcolcProc : PROCESS(internalcolc, colc)
BEGIN
		
	IF colc > D THEN
		IF internalcolc < D THEN
			internalcolc_next <= internalcolc + 1;
		ELSE 
			internalcolc_next <= to_unsigned(1, ImageWidthInBits);
		END IF;
	ELSE 
		internalcolc_next <= internalcolc;
	END IF;
END PROCESS;

--Assign the 4 small vectors to the big output vector 
outproc : PROCESS(nminus1, n, nplus1, prevdispexp)
BEGIN
	FOR i IN 1 TO 5 LOOP
		expandout(i) <= nminus1(i);
		expandout(i+5) <= n(i);
		expandout(i+10) <= nplus1(i);
		expandout(i+15) <= prevdispexp(i);
	END LOOP;
END PROCESS;


END behaviour;























