-- Receives minimum index from find min module of absolute differences vector. Uses houghline vote 
-- pipeline to vote D values and hence, find the most often occuring index i.e. the most dominating disparity.
-- Calculates also the maximum vote index on the fly (WOW!)


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.allconstantsPack.ALL;

entity houghmodule_new is
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		colc : IN unsigned(ImageWidthInBits - 1 downto 0);
		index : IN unsigned(7 downto 0);		--index of minium 
		maxindex : OUT unsigned(DinBits - 1 downto 0)
	);
end houghmodule_new;

architecture Behavioral of houghmodule_new is

signal houghline, houghline_next : pixelBitWidthpipeLenDType := (OTHERS => (OTHERS => '0'));	--Internal processing pipeline
signal internalcolc, internalcolc_next : unsigned(DinBits - 1 downto 0) := (OTHERS => '0');		--Internal Column counter to reintialize to 0

signal currentmaxval, currentmaxval_next : unsigned(DinBits - 1 downto 0) := (OTHERS => '0');
signal currentmaxind, currentmaxind_next : unsigned(DinBits - 1 downto 0) := (OTHERS => '0');

begin
	clkProc : PROCESS(clk, reset)
	BEGIN
		IF reset = '1' THEN
			houghline <= (OTHERS => (OTHERS => '0'));
			internalcolc <= to_unsigned(1,DinBits);
			currentmaxval <= (OTHERS => '0');
			currentmaxind <= (OTHERS => '0');
		ELSIF clk'event AND clk = '1' THEN
			houghline <= houghline_next;
			internalcolc <= internalcolc_next;
			currentmaxval <= currentmaxval_next;
			currentmaxind <= currentmaxind_next;
		END IF;
	END PROCESS;

	intcolcounterProc : PROCESS(internalcolc)
	BEGIN
		internalcolc_next <= internalcolc;
		
		--IF colc > D THEN
			IF internalcolc < D THEN
				internalcolc_next <= internalcolc + 1;
			ELSE
				internalcolc_next <= to_unsigned(1, DinBits);
			END IF;
		--ELSE
			--internalcolc_next <= to_unsigned(1, DinBits);
		--END IF;
	END PROCESS;
	
	combProc : PROCESS(index, colc, houghline, internalcolc, currentmaxval, currentmaxind)
	BEGIN

		currentmaxval_next <= currentmaxval;
		currentmaxind_next <= currentmaxind;
		houghline_next <= houghline;
		maxindex <= (OTHERS => '0');
		
		--Action only, when x is bigger than D, ie. spare left border
		IF colc > D THEN
			IF index >= 1 AND index <= D THEN
				houghline_next(to_integer(index)) <= houghline(to_integer(index)) + 1;	--Vote
				
				IF houghline(to_integer(index)) = currentmaxval THEN		--berprfe, ob die zu erhhende Stelle bereits einen Maximalwert beinhaltet. Wenn ja, erhhre den Maximalwert ebenfalls und setzt neuen index
					currentmaxval_next <= currentmaxval + 1;		--Ausnutzung der Tatache, dass aufeinander folgende houghline-WErte maximal um eins grer sein knnen
					currentmaxind_next <= index(DinBits - 1 downto 0);						--Aktualisierung des Index auf neues Maximum
				END IF;
			ELSE
				houghline_next <= houghline;	--If not vote, prevent latch
			END IF;
			
			--Check, if the hough line must be (re-)initialized
			IF internalcolc = 1 THEN
				houghline_next <= (OTHERS => (OTHERS => '0'));	--Init houghline
				houghline_next(to_integer(index)) <= to_unsigned(1, pixelBitWidth);	--but process current index
				currentmaxval_next <= (OTHERS => '0');
				currentmaxind_next <= (OTHERS => '0');
				maxindex <= currentmaxind;
			END IF;
		ELSE
			houghline_next <= (OTHERS => (OTHERS => '0'));
		END IF;
	
	END PROCESS;
	
END Behavioral;









