library IEEE;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.constPack.ALL;

--Kann nicht synthetisiert werden, da Werte aus Fifo gelesen werden, miteinander addiert werden und wieder 
--im das selbe Fifo geschrieben werden sollen. Das explodiert irgendwann. 
--Erkenntis: Inferrierung vieler Block-Rams (160 Stck,  1242 8 Bit-Eintrgen) in Schleifen sowie Verschaltung dieser ohne Probleme mglich! 
--Vielleicht kann es ein bestimmten DP-BRAM Modus geben wie read first,
 
ENTITY main is
PORT(
	clk : in std_logic;
	reset : in std_logic;
	costs : in dVectorType;
	--outval : out unsigned(7 downto 0)
	vecsigout : out vecsigType
);
	
end main;

ARCHITECTURE Behavioral of main is

--signal colc, colc_next : unsigned(ImageWidthInBits - 1 downto 0);

signal vecsig, vecsig_next : vecsigType;

--signal outsig, outsig_next : unsigned(7 downto 0);

signal fifotopipe : vecsigType;

type procpipeType IS ARRAY(NATURAL RANGE 1 TO D, NATURAL RANGE 1 TO 3) OF unsigned(7 downto 0);
signal procpipe, procpipe_next : procpipeType;

CONSTANT DATAWIDTH : integer := 10;

COMPONENT fifowrapper
	generic (
		DATAWIDTH : integer := 10;	--data range from 0 to (2^DATAWIDTH) - 1
		ADDRLEN : integer := WI		--FIFO delays data by ADDRLEN cycles. For 3x3, -2 must be added (nobody knows why)
	);
	port (
		clk : in std_logic;
		reset : in std_logic;
		wdata : in std_logic_vector((DATAWIDTH - 1) downto 0);
		rdatatop : out unsigned((DATAWIDTH - 1) downto 0)
	);
END COMPONENT;

BEGIN

--Fifo wrapper ports: clk, reset, writedata, readdata
GENERATE_FIFO:
	FOR i IN 1 TO D GENERATE
		myfifox : fifowrapper PORT MAP(
			clk => clk,
			reset => reset, 
			wdata => std_logic_vector(vecsig(i)), 
			rdatatop => fifotopipe(i)
		);
	END GENERATE GENERATE_FIFO;

clkProc : process (clk, reset)
BEGIN
	IF reset = '1' THEN
		vecsig <= (OTHERS => (OTHERS => '0'));
		procpipe <= (OTHERS => (OTHERS => (OTHERS => '0')));
		--colc <= to_unsigned(1,ImageWidthInBits);
		--outsig <= (OTHERS => '0');
	ELSIF clk'event AND clk = '1' THEN
		procpipe <= procpipe_next;
		vecsig <= vecsig_next;
		--colc <= colc_next;
		--outsig <= outsig_next;
	END IF;
END PROCESS;

--costs: 160 values
inProc : PROCESS(costs, procpipe, vecsig, fifotopipe)
variable tmp1, tmp2 : unsigned(8 downto 0);
BEGIN
	
	vecsig_next <= vecsig;
	
	FOR a IN 1 TO 160 LOOP
		procpipe_next(a, 1) <= procpipe(a,2);
		procpipe_next(a, 2) <= procpipe(a,3);
		procpipe_next(a, 3) <= fifotopipe(a);
	END LOOP;
	
	FOR i IN 1 TO 160 LOOP
		tmp1 := ('0' & costs(i)) + ('0' & procpipe(i,1));		--Add 2 8 Bit values
		tmp2 := ('0' & procpipe(i,2)) + ('0' & procpipe(i,3));
		vecsig_next(i) <=  ('0' & tmp1) + ('0' & tmp2);
	END LOOP;
	
	-- IF colc < WI THEN
		-- colc_next <= colc + 1;
	-- ELSE
		-- colc_next <= to_unsigned(1,ImageWidthInBits);
	-- END IF;
	
END PROCESS;

-- outProc : PROCESS(vecsig, outsig)

-- type medvec1Type IS ARRAY(NATURAL RANGE 1 TO 80) OF unsigned(7 downto 0);
-- type medvec2Type IS ARRAY(NATURAL RANGE 1 TO 40) OF unsigned(7 downto 0);
-- type medvec3Type IS ARRAY(NATURAL RANGE 1 TO 20) OF unsigned(7 downto 0);
-- type medvec4Type IS ARRAY(NATURAL RANGE 1 TO 10) OF unsigned(7 downto 0);
-- type medvec5Type IS ARRAY(NATURAL RANGE 1 TO 5) OF unsigned(7 downto 0);
-- type medvec6Type IS ARRAY(NATURAL RANGE 1 TO 3) OF unsigned(7 downto 0);
-- type medvec7Type IS ARRAY(NATURAL RANGE 1 TO 2) OF unsigned(7 downto 0);

-- variable medvec1 : medvec1Type := (OTHERS => (OTHERS => '1'));
-- variable medvec2 : medvec2Type := (OTHERS => (OTHERS => '1'));
-- variable medvec3 : medvec3Type := (OTHERS => (OTHERS => '1'));
-- variable medvec4 : medvec4Type := (OTHERS => (OTHERS => '1'));
-- variable medvec5 : medvec5Type := (OTHERS => (OTHERS => '1'));
-- variable medvec6 : medvec6Type := (OTHERS => (OTHERS => '1'));
-- variable medvec7 : medvec7Type := (OTHERS => (OTHERS => '1'));

-- BEGIN
	
	-- outsig_next <= outsig;
	
	-- --Reduce 160 values to 80 by selecting the minimum of two neighbors
	-- FOR i IN 1 TO 80 LOOP
		-- IF vecsig(i * 2) < vecsig((i * 2) - 1) THEN
			-- medvec1(i) := vecsig(i * 2);
		-- ELSE
			-- medvec1(i) := vecsig((i * 2) - 1);
		-- END IF;
	-- END LOOP;
	
	-- --REduce 80 values to 40
	-- FOR i IN 1 TO 40 LOOP
		-- IF medvec1(i * 2) < medvec1((i * 2) - 1) THEN
			-- medvec2(i) := medvec1(i * 2);
		-- ELSE
			-- medvec2(i) := medvec1((i * 2) - 1);
		-- END IF;
	-- END LOOP;
	
	-- --40 to 20
	-- FOR i IN 1 TO 20 LOOP
		-- IF medvec2(i * 2) < medvec2((i * 2) - 1) THEN
			-- medvec3(i) := medvec2(i * 2);
		-- ELSE
			-- medvec3(i) := medvec2((i * 2) - 1);
		-- END IF;
	-- END LOOP;
	
	-- --20 to 10
	-- FOR i IN 1 TO 10 LOOP
		-- IF medvec3(i * 2) < medvec3((i * 2) - 1) THEN
			-- medvec4(i) := medvec3(i * 2);
		-- ELSE
			-- medvec4(i) := medvec3((i * 2) - 1);
		-- END IF;
	-- END LOOP;
	
	-- --10 to 5
	-- FOR i IN 1 TO 5 LOOP
		-- IF medvec4(i * 2) < medvec4((i * 2) - 1) THEN
			-- medvec5(i) := medvec4(i * 2);
		-- ELSE
			-- medvec5(i) := medvec4((i * 2) - 1);
		-- END IF;
	-- END LOOP;
	
	-- --5 to 3
	-- IF medvec5(1) < medvec5(2) THEN
		-- medvec6(1) := medvec5(1);
	-- ELSE
		-- medvec6(1) := medvec5(2);
	-- END IF;
	
	-- IF medvec5(3) < medvec5(4) THEN
		-- medvec6(2) := medvec5(3);
	-- ELSE
		-- medvec6(2) := medvec5(4);
	-- END IF;
	
	-- medvec6(3) := medvec5(5);
		
	-- --3 to 2
	-- IF medvec6(1) < medvec6(2) THEN
		-- medvec7(1) := medvec6(1);
	-- ELSE
		-- medvec7(1) := medvec6(2);
	-- END IF;
	
	-- medvec7(2) := medvec6(3);
	
	-- --select the min of the last 2 values:
	-- IF medvec7(1) < medvec7(2) THEN
		-- outsig_next <= medvec7(1);
	-- ELSE
		-- outsig_next <= medvec7(2);
	-- END IF;
	
-- END PROCESS;

vecsigout <= vecsig;
--outval <= outsig;

end Behavioral;























