library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.math_real.ALL;

--Reads data from external addr. If we is true, writes data to an external addr. 
--Can be used as FIFO, if the readdr is 1 biger than wraddrs. Wrap around must be controlled from externally.  

entity fifodata is
generic (
	DATA : integer := 8;			--use this as data bit width
	ADDRLEN : integer := 127		--this is the number of elements i.e. ram is able to store ADDRLEN data values!
);		

port (
	clk : in std_logic;
	we : in std_logic;		--write enable
	waddr : in std_logic_vector(integer(ceil(log2(real(ADDRLEN))))-1 downto 0);	--0 .. 128, 8 bit
	wdata : in std_logic_vector((DATA - 1) downto 0);
	raddr : in std_logic_vector(integer(ceil(log2(real(ADDRLEN))))-1 downto 0);
	--rdata : out std_logic_vector((DATA - 1) downto 0)
	rdata : out unsigned((DATA - 1) downto 0)
);
end fifodata;

architecture Behavioral of fifodata is

--Declaration of type and signal of a 2^ADDR element RAM
--with each element being DATA bit wide.
type ram_t is array (0 TO ADDRLEN) of std_logic_vector(DATA-1 downto 0);
signal ram : ram_t := (OTHERS => (OTHERS => '0'));

begin

--process for read and write operation.
PROCESS(clk)
BEGIN
   IF clk'event AND clk = '1' then
		IF (we = '1') THEN
			ram(to_integer(unsigned(waddr))) <= wdata;
		END IF;
		rdata <= unsigned(ram(to_integer(unsigned(raddr))));
   END IF;
END PROCESS;

END Behavioral;
